REBOL []

random/seed now/time/precise 

zeropad: func [value n][
	head insert/dup value: form value #"0" n - length? value
]

ceil: func [n [number!]][
	if integer? n [return n]
	to integer! n + 0.99999999999999
]

mark: context [
	selected: func [key field [word!] default][
		if key = any [request/content/:field :default][prin " selected "]
	]
	
	error: func [field [word!]][
		if find invalid :field [prin {class="error"}]
	]
]

encode-pass: func [pass [string!]][
	enbase/base checksum/method pass 'md5 16
]

pre-chars: charset "^/^-"
pre-encode: func [data [string!]][
	convert data [pre-chars][
		either value = "^/" [
			"<br/>"
		][
			"&nbsp;&nbsp;&nbsp;&nbsp;"
		]
	]
]

not-ws: complement charset "^/^-^M "
activate-links: func [data [string!] /local s e link][
	parse/all data [
		any [
			s: "http://" some not-ws e: (
				change/part s link: rejoin [
					{<a href="} link: copy/part s e {">}
					link 
					"</a>"
				] e
				s: skip s length? link
			) :s
			| skip
		]
	]
	data
]

coltitle: func [pos name /local current?][
	if current?: pos = abs order [
		prin {<img src="img/} 
		prin pick ["up" "down"] positive? order
		prin {.png" align="right">}
	]
	prin {<a href="view-tickets.rsp?orderby=}
	prin any [all [current? negate order] pos]
	prin {">}
	prin say name
	prin {</a>}
]

context [
	labels: make hash! 48
	foreach [w c][
		added		-
		removed		-
		modified	-
		summary		-
		description	-
		code		-
		severity	severities
		status		statuses
		resolution	resolv
		priority	priorities
		reproduce	reproduces
		version		-
		fixedin		-
		category	-
		comment		-
		ticket		-
	][
		repend labels [w c uppercase/part form w 1]
	]
	poke labels 11 * 3 "Reproductibility"
	
	set 'log-encode func [w [word!]][
		divide 2 + index? find/skip labels w 3 3
	]
	set 'log-decode func [fid [integer!] /value vid /local w][
		say either all [
			value
			'- <> w: pick labels 3 * fid - 1 
		][
			select do-sql/flat 'bugs w vid
		][
			pick labels 3 * fid
		]
	]	
]


set 'form-time func [time [time!]][
	copy/part either time/hour < 10 [
		head insert mold time #"0"
	][
		mold time
	] 5
]

set 'to-UTC func [date [date!]][date - date/zone]

make-vkey: has [key][
	key: make string! 16
	loop 16 [append key #"@" + random 26]
	key: checksum/method key 'md5
	enbase/base key 16
]

; ====== Email management ======

system/user/name: config/name
set-net reduce [no-reply@curecode.org config/STMP-server]

send-confirmation: func [
	spec [block!] vkey [string!]
	/local url login pass template
][
	url: rejoin [
		request/headers/Host
		either request/server-port = 80 [""][join ":" request/server-port]
		request/web-app
		"/validate.rsp?id=" url-encode spec/login 
		"&key=" vkey
	]
	login: spec/login
	pass: spec/pass
	template: read join locale/get-path %email-activation.tpl
	replace template "$url" url
	replace template "$login" login
	replace template "$pass" pass
	
	send/subject spec/email template
		rejoin ["[" config/name "] " say "Account activation"]
]

send-reset-URL: func [
	spec [block!] vkey [string!]
	/local url template
][
	url: rejoin [
		request/headers/Host
		either request/server-port = 80 [""][join ":" request/server-port]
		request/web-app
		"/reset.rsp?id=" url-encode spec/login 
		"&key=" vkey
	]
	template: read join locale/get-path %email-pass-reset.tpl
	replace template "$url" url
	
	send/subject spec/email template
		rejoin ["[" config/name "] " say "Password reset confirmation"]
]
